package dev.fyloz.colorrecipesexplorer.rest

import dev.fyloz.colorrecipesexplorer.model.touchupkit.TouchUpKitOutputDto
import dev.fyloz.colorrecipesexplorer.model.touchupkit.TouchUpKitSaveDto
import dev.fyloz.colorrecipesexplorer.model.touchupkit.TouchUpKitUpdateDto
import dev.fyloz.colorrecipesexplorer.service.touchupkit.TouchUpKitService
import org.springframework.core.io.ByteArrayResource
import org.springframework.http.MediaType
import org.springframework.http.ResponseEntity
import org.springframework.security.access.prepost.PreAuthorize
import org.springframework.web.bind.annotation.*
import javax.validation.Valid

const val TOUCH_UP_KIT_CONTROLLER_PATH = "/api/touchupkit"

@RestController
@RequestMapping(TOUCH_UP_KIT_CONTROLLER_PATH)
@PreAuthorize("hasAuthority('VIEW_TOUCH_UP_KITS')")
class TouchUpKitController(
    private val touchUpKitService: TouchUpKitService
) {
    @GetMapping
    fun getAll() =
        ok(touchUpKitService.getAllForOutput())

    @GetMapping("{id}")
    fun getById(@PathVariable id: Long) =
        ok(touchUpKitService.getByIdForOutput(id))

    @PostMapping
    @PreAuthorize("hasAuthority('EDIT_TOUCH_UP_KITS')")
    fun save(@Valid @RequestBody touchUpKit: TouchUpKitSaveDto) =
        created<TouchUpKitOutputDto>(TOUCH_UP_KIT_CONTROLLER_PATH) {
            with(touchUpKitService) {
                save(touchUpKit).toOutput()
            }
        }

    @PutMapping
    @PreAuthorize("hasAuthority('EDIT_TOUCH_UP_KITS')")
    fun update(@Valid @RequestBody touchUpKit: TouchUpKitUpdateDto) =
        noContent {
            touchUpKitService.update(touchUpKit)
        }

    @DeleteMapping("{id}")
    @PreAuthorize("hasAuthority('EDIT_TOUCH_UP_KITS')")
    fun deleteById(@PathVariable id: Long) =
        noContent {
            touchUpKitService.deleteById(id)
        }

    @GetMapping("pdf")
    fun getJobPdf(@RequestParam project: String): ResponseEntity<ByteArrayResource> {
        with(touchUpKitService.generateJobPdfResource(project)) {
            return ResponseEntity.ok()
                .header("Content-Disposition", "filename=TouchUpKit_$project.pdf")
                .contentLength(this.contentLength())
                .contentType(MediaType.APPLICATION_PDF)
                .body(this)
        }
    }
}
